#pragma once
#include <PSX/GPU/gpu.hpp>
#include <PSX/GPU/make_gpu_primitives.hpp>

namespace ScreenCenter {
    using namespace JabyEngine;
    
    class Frame : public GPU::internal::LinkedElementCreator<Frame> {
    private:
        GPU::TILE            top_left[2];
        GPU::TILE            top_right[2];
        GPU::TILE            bottom_left[2];
        GPU::TILE            bottom_right[2];
        GPU::LINE_G_MULTI<5> border;
        GPU::LINE_G_SINGLE   cross[2];

    public:
        static constexpr Frame::Linked create() {
            constexpr auto BaseColor = GPU::Color24::from_rgb(0x1D, 0xA0, 0xA3);
            constexpr auto Size      = Make::SizeI16(64, 16);

            Frame frame;

            frame.top_left[0] = Make::TILE(Make::AreaI16(0, 0, Size.width, Size.height), BaseColor);
            frame.top_left[1] = Make::TILE(Make::AreaI16(0, 0, Size.height, Size.width), BaseColor);

            frame.top_right[0] = Make::TILE(Make::AreaI16(GPU::Display::Width - Size.width, 0, Size.width, Size.height), BaseColor);
            frame.top_right[1] = Make::TILE(Make::AreaI16(GPU::Display::Width - Size.height, 0, Size.height, Size.width), BaseColor);

            frame.bottom_left[0] = Make::TILE(Make::AreaI16(0, GPU::Display::Height - Size.width, Size.height, Size.width), BaseColor);
            frame.bottom_left[1] = Make::TILE(Make::AreaI16(0, GPU::Display::Height - Size.height, Size.width, Size.height), BaseColor);

            frame.bottom_right[0] = Make::TILE(Make::AreaI16(GPU::Display::Width - Size.height, GPU::Display::Height - Size.width, Size.height, Size.width), BaseColor);
            frame.bottom_right[1] = Make::TILE(Make::AreaI16(GPU::Display::Width - Size.width, GPU::Display::Height - Size.height, Size.width, Size.height), BaseColor);
            
            frame.border = Make::LINE_G(
                GPU::ColorVertex{GPU::Color24::Red(),    Make::Vertex(0,                       0)},
                GPU::ColorVertex{GPU::Color24::Green(),  Make::Vertex(0,                       GPU::Display::Height - 1)},
                GPU::ColorVertex{GPU::Color24::Blue(),   Make::Vertex(GPU::Display::Width - 1, GPU::Display::Height - 1)},
                GPU::ColorVertex{GPU::Color24::Yellow(), Make::Vertex(GPU::Display::Width - 1, 0)},
                GPU::ColorVertex{GPU::Color24::Red(),    Make::Vertex(0,                       0)}
            );

            frame.cross[0] = Make::LINE_G(frame.border[0], frame.border[2]);
            frame.cross[1] = Make::LINE_G(frame.border[3], frame.border[1]);
            return frame.linked();
        }
    };
}