#pragma once
#include "../Overlay/overlay.hpp"
#include "file_types.hpp"

namespace JabyEngine {
    enum struct CDFileType : uint8_t {
        SimpleTIM = 0,
        CopyTo,
    };

    #pragma pack(push, 1)
    struct CDFile {
        union Payload {
            uint32_t  raw;
            SimpleTIM simple_tim;
            CopyTo    copy_to;
            Overlay   overlay;
        };

        uint8_t    rel_lba_idx;
        CDFileType type;
        Payload    payload;

        static_assert(sizeof(Payload) == sizeof(uint32_t));
    };
    #pragma pack(pop)

    struct CDFileBuilder {
        static constexpr CDFile simple_tim(uint8_t rel_lba_idx, SimpleTIM simple_tim) {
            return CDFile{.rel_lba_idx = rel_lba_idx, .type = CDFileType::SimpleTIM, .payload = {.simple_tim = simple_tim}};
        }

        static constexpr CDFile copy_to(uint8_t rel_lba_idx, uint32_t* dst) {
            return CDFile{.rel_lba_idx = rel_lba_idx, .type = CDFileType::CopyTo, .payload = {.copy_to = CopyTo{dst}}};
        }

        static constexpr CDFile overlay(uint8_t rel_lba_idx, uint32_t* overlay_dst) {
            return CDFile{.rel_lba_idx = rel_lba_idx, .type = CDFileType::CopyTo, .payload = {.overlay = Overlay{overlay_dst}}};
        }
    };
}