#include <PSX/System/IOPorts/spu_io.hpp>
#include <stdio.h>
#include <limits.h>

namespace JabyEngine {
    namespace boot {
        namespace SPU {
            using namespace JabyEngine;
            using namespace SPU_IO;
            
            static void clear_main_volume() {
                static constexpr auto StartVol = SweepVolume::from(SweepVolume::VolumeEnable, SweepVolume::Volume.with(static_cast<int16_t>(I16_MAX >> 2)));

                MainVolume::Left.write(StartVol);
                MainVolume::Right.write(StartVol);
            }

            static void clear_cd_and_ext_audio_volume() {
                CDVolume::Left.write({0});
                CDVolume::Right.write({0});

                ExternalAudioInputVolume::Left.write({0});
                ExternalAudioInputVolume::Right.write({0});
            }

            static void clear_control_register() {
                ControlRegister.write({0});
            }

            static void clear_voice() {
                for(auto& voice : SPU_IO::Voice) {
                    voice.volumeLeft.write({0});
                    voice.volumeRight.write({0});
                    voice.sampleRate.write({0});
                    voice.ad.write({0});
                    voice.sr.write({0});
                    voice.currentVolume.write({0});
                    
                    voice.adr.write(Voice::start_adr());
                    voice.repeatAdr.write(Voice::start_adr());
                }
            }

            static void clear_pmon() {
                SPU_IO::PMON.write({0});
            }

            static void clear_noise_and_echo() {
                SPU_IO::NON.write({0});
                SPU_IO::EON.write({0});
            }

            static void clear_reverb() {
                Reverb::Volume::Left.write({0});
                Reverb::Volume::Right.write({0});
                Reverb::WorkAreaAdr.write({0});
            }

            static void setup_control_register() {
                static constexpr auto SetupValue = ControlRegister::from(ControlRegister::Enable, ControlRegister::Unmute, ControlRegister::CDAudioEnable);

                SPU_IO::ControlRegister.write(SetupValue);
            }

            static void setup_data_transfer_control() {
                static constexpr struct DataTransferControl RequiredValue{(2 << 1)};
                
                DataTransferControl.write(RequiredValue);
            }

            static void wait_voices() {
                static constexpr SimpleVolume Treshhold{static_cast<int16_t>(I16_MAX*0.03)};

                try_again:
                for(const auto& voice : SPU_IO::Voice) {
                    if(voice.currentVolume.read() > Treshhold) {
                        goto try_again;
                    }
                }
            }

            void stop_voices() {
                SPU_IO::Key::Off.write({UI32_MAX});
            }

            void setup() {
                wait_voices();

                clear_main_volume();
                clear_cd_and_ext_audio_volume();
                clear_control_register();
                clear_voice();
                clear_pmon();
                clear_noise_and_echo();
                clear_reverb();

                setup_data_transfer_control();
                setup_control_register();
            }
        }
    }
}