#include <PSX/GTE/gte.hpp>
#include <math.h>

int32_t sin(int32_t value) {
    static constexpr int32_t qN = 13;
    static constexpr int32_t qA = 12;
    static constexpr int32_t B  = 19900;
    static constexpr int32_t C  = 3516;

    const auto c = value << (30 - qN);          // Semi-circle info into carry.

    value -= 1<<qN;                             // sine -> cosine calc
    value  = value       << (31 - qN);          // Mask with PI
    value  = value       >> (31 - qN);          // Note: SIGNED shift! (to qN)
    value  = value*value >> (2*qN - 14);        // x=x^2 To Q14

    auto result = B - (value*C >> 14);              // B - x^2*C
         result = (1 << qA) - (value*result >> 16); // A - x^2*(B-x^2*C)

    return c >= 0 ? result : -result;
}

int32_t cos(int32_t value) {
    return sin(value + (FULL_CIRCLE/4));
}

namespace JabyEngine {
    namespace GTE {
        MATRIX& multiply_matrix(const MATRIX& m0, const MATRIX& m1, MATRIX& result) {
            set_rot_matrix(m0);
            
            JabyEngine::GTE::ldclmv(m1, 0);
            JabyEngine::GTE::rtir();
            JabyEngine::GTE::stclmv(result, 0);
            
            JabyEngine::GTE::ldclmv(m1, 1);
            JabyEngine::GTE::rtir();
            JabyEngine::GTE::stclmv(result, 1);
            
            JabyEngine::GTE::ldclmv(m1, 2);
            JabyEngine::GTE::rtir();
            JabyEngine::GTE::stclmv(result, 2);

            return result;
        }

        void set_matrix(const MATRIX& matrix) {
            set_rot_matrix(matrix);
            set_trans_matrix(matrix);
        }

        MATRIX get_matrix() {
            MATRIX matrix;

            get_rot_matrix(matrix);
            get_trans_matrix(matrix);
            return matrix;
        }

        void push_matrix() {
            // FIXME: Implement this
        }

        void push_matrix(const MATRIX& matrix) {
            push_matrix();
            set_matrix(matrix);
        }

        void pop_matrix() {
            // FIXME: Implement this
        }
    }
}